import sys

from gempy_viewer.modules.plot_3d.vista import GemPyToVista
from .API import *
import gempy
__all__ = ['plot_2d', 'plot_3d', 'plot_section_traces', 'plot_topology', 'plot_stereonet']


# Assert at least pyton 3.10
assert sys.version_info[0] >= 3 and sys.version_info[1] >= 10, "GemPy requires Python 3.10 or higher"
__version__ = gempy.__version__

if __name__ == '__main__':
    pass



import warnings

from ._plot_2d_API import plot_2d, plot_section_traces, plot_topology 
from ._plot_LiquidEarth import plot_to_liquid_earth
# check if pyvista is installed
try:
    from ._plot_3d_API import plot_3d
except ImportError:
    warnings.warn("The pyvista package is required to plot 3D models. ")
from ._plot_others import plot_stereonet


def require_liquid_earth_api():
    try:
        import liquid_earth_api
    except ImportError:
        raise ImportError("The liquid_earth_api package is required to run this function.")
    return liquid_earth_api

def require_gempy_plugins():
    try:
        import gempy.plugins
    except ImportError:
        raise ImportError("The gempy.plugins package is required to run this function.")
    return gempy.plugins


def require_pyvista():
    try:
        import pyvista as pv
    except ImportError:
        raise ImportError("The pyvista package is required to run this function.")
    return pv

def require_scipy():
    try:
        import scipy
    except ImportError:
        raise ImportError("The scipy package is required to run this function.")
    return scipy

def require_skimage():
    try:
        import skimage
    except ImportError:
        raise ImportError("The skimage package is required to run this function.")
    return skimage
    

def require_pandas():
    try:
        import pandas as pd
    except ImportError:
        raise ImportError("The pandas package is required to run this function.")
    return pd


from typing import Optional

import gempy
from gempy_viewer.optional_dependencies import require_liquid_earth_api


def plot_to_liquid_earth(
        geo_model: gempy.data.GeoModel, space_name: str,
        file_name: str = "gempy_model", user_token: Optional[str] = None, grab_link=True,
        make_new_space: bool = False
):
    # if user_token is None Try to grab it from the environment
    liquid_earth_api = require_liquid_earth_api()  # ! Order matters

    if user_token is None:
        import os
        user_token = os.environ.get('LIQUID_EARTH_API_TOKEN')
        if user_token is None:
            raise ValueError("No user token provided and no token found in the environment")

    if make_new_space:
        result = liquid_earth_api.upload_mesh_to_new_space(
            space_name=space_name,
            data=geo_model.solutions.raw_arrays.meshes_to_subsurface(),
            file_name=file_name,
            token=user_token,
            grab_link=grab_link
        )
    else:
        result = liquid_earth_api.upload_mesh_to_existing_space(
            space_name=space_name,
            data=geo_model.solutions.raw_arrays.meshes_to_subsurface(),
            file_name=file_name,
            token=user_token,
            grab_link=grab_link
        )
    return result



import os
from os import path

from setuptools import setup, find_packages


def read_requirements(file_name, base_path=""):
    # Construct the full path to the requirements file
    full_path = os.path.join(base_path, file_name)
    requirements = []
    with open(full_path, "r", encoding="utf-8") as f:
        for line in f:
            # Strip whitespace and ignore comments
            line = line.strip()
            if line.startswith("#") or not line:
                continue

            # Handle -r directive
            if line.startswith("-r "):
                referenced_file = line.split()[1]  # Extract the file name
                # Recursively read the referenced file, making sure to include the base path
                requirements.extend(read_requirements(referenced_file, base_path=base_path))
            else:
                requirements.append(line)

    return requirements


setup(
    name='gempy_viewer',
    # version=version,
    packages=find_packages(),
    url='',
    license='EUPL',
    author='Miguel de la Varga',
    author_email="miguel@terranigma-solutions.com",
    description='Viewer for the geological modeling package GemPy',
    install_requires=read_requirements("requirements.txt", "requirements"),
    extras_require={
            "opt": read_requirements("optional_requirements.txt", "requirements"),
            "dev": read_requirements("dev-requirements.txt" , "requirements"),
    },
    classifiers=[
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Science/Research',
            'Topic :: Scientific/Engineering :: Visualization',
            'Topic :: Scientific/Engineering :: GIS',
            'Programming Language :: Python :: 3.10'
    ],
    python_requires='>=3.10',
    setup_requires=['setuptools_scm'],
    use_scm_version={
            "root"            : ".",
            "relative_to"     : __file__,
            "write_to"        : path.join("gempy_viewer", "_version.py"),
            "fallback_version": "3.0.0"
    },
    zip_safe=False
)
